/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved

	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.

	This file is under Perforce control
	$Id: //depot/express/fcs992/include/avs/rp.h#1 $
*/
#ifndef RP_INCLUDED
#define RP_INCLUDED

#include <avs/comm.h>
#include <avs/event.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef int  (*RPpfi)();
typedef void (*RPpfv)();

typedef struct _RPcall {
   void (*func) PTARGS((int, COMMconn *, char *));
   int code;
   char *arg;
   struct _RPcall *next;
} RPcall;

typedef struct _RPcall_list {
   int hash_size;
   RPcall **hash_table;
} RPcall_list;

typedef struct _RPserver {
   COMMconn  *conn;
   RPcall_list *list;
   void (*destroy_func) PTARGS((void *));
   void *destroy_arg;
   unsigned int destroyed:1;
} RPserver;

#define RP_SERVER_ARG(C)	(((RPserver *)(C)->destroy_arg)->destroy_arg)

typedef struct _RPpack_val {
   unsigned int scalar:1,type:3,sent:1;
   union {
      union {
	 int 	ival;
	 short 	sval;
	 char 	bval;
	 float 	fval;
	 double dval;
      } s;
      struct _RParray {
         int offset, length;
	 char *ptr;
      } a;
   } v;
} RPpack_val;

typedef struct _RPpacket {
   int err, sync;
   COMMconn  *conn;
   int nvals, nmaxvals;
   RPpack_val *vals;
} RPpacket;

#define RPinteger_array_read(C,A,L) RParray_read(C,COMM_TYPE_INT,(char **)A,L)
#define RPbyte_array_read(C,A,L)   RParray_read(C,COMM_TYPE_BYTE,A,L)
#define RPfloat_array_read(C,A,L)  RParray_read(C,COMM_TYPE_FLOAT,(char **)A,L)
#define RPdouble_array_read(C,A,L) RParray_read(C,COMM_TYPE_DOUBLE,(char **)A,L)
#define RPshort_array_read(C,A,L)  RParray_read(C,COMM_TYPE_SHORT,(char **)A,L)

#define RPinteger_read(C,A) RPprim_read(C,COMM_TYPE_INT,(char *)A)
#define RPbyte_read(C,A)    RPprim_read(C,COMM_TYPE_BYTE,A)
#define RPfloat_read(C,A)   RPprim_read(C,COMM_TYPE_FLOAT,(char *)A)
#define RPdouble_read(C,A)  RPprim_read(C,COMM_TYPE_DOUBLE,(char *)A)
#define RPshort_read(C,A)   RPprim_read(C,COMM_TYPE_SHORT,(char *)A)

RPserver 	*RPinit_server_conn PTARGS( (COMMconn *, RPcall_list *list,
					     void (*func)(void *), void *) );
void 		RPdestroy_server PTARGS( (RPserver *) );
void		RPdispatch PTARGS( (char *) );
void		RPinit_client_conn PTARGS( (COMMconn *, void (*func)(void *),
					    void *) ); 
int		RPset_dispatch_hold PTARGS( (int val) );
RPcall_list     *RPcreate_call_list PTARGS( (int) );
void		RPdefine_handler PTARGS( (RPcall_list *,unsigned int,
					  void (*)(),char *));
int		RPsend_packet PTARGS( (COMMconn *, RPpacket *, int sync) );
RPpacket	*RPnew_packet PTARGS( (int));
void		RPfree_packet PTARGS( (RPpacket *) );
void 		RPadd_array_packet PTARGS((RPpacket *, int, char *, int));
void 		RPadd_int_packet PTARGS((RPpacket *, int));
void 		RPadd_float_packet PTARGS((RPpacket *, float));
void 		RPadd_double_packet PTARGS((RPpacket *, double));
void		RPadd_string_packet PTARGS((RPpacket *, char *));
void		RPerror PTARGS( (char *) );
void		RPinternal_error PTARGS( (char *) );
RPcall		*RPlookup_handler PTARGS( (RPcall_list *, unsigned int) );
int		RPprim_read PTARGS((COMMconn *, int, char *));
int		RParray_read PTARGS((COMMconn *, int, char **, int));
int		RPstring_read PTARGS((COMMconn *conn, char **str));

#ifdef __cplusplus
} /* End of extern "C" */
#endif

#endif
